///<ObjektNummer>1935</ObjektNummer>

#if !defined(h_Ethernet)
#define h_Ethernet

// Defines
#define F_CPU 20000000
#define MCU ATMEGA644P

#include <avr\io.h>
#include "ENC28J60.h"

#include "Mx8_Timer.h"
#include "M8_Mx8_SPI.h"

// Makro zur einfachen Definition der IP-Adresse
#define IP(a,b,c,d) ((uint32_t)a+((uint32_t)b<<8)+((uint32_t)c<<16)+((uint32_t)d<<24))
#define setMyMacAddress(a,b,c,d,e,f) myMacAddress[0]=a;myMacAddress[1]=b;myMacAddress[2]=c;myMacAddress[3]=d;myMacAddress[4]=e;myMacAddress[5]=f;

/////////// Ethernet ///////////////////////////
struct EthernetHeader {
	unsigned char ETH_destMac[6];
	unsigned char ETH_sourceMac[6];
	unsigned int ETH_typefield;
};
#define ETHERNET_MIN_PACKET_LENGTH	0x3C
////////// ARP //////////////////////////////////
struct ArpHeader {
    unsigned int HWtype;                
    unsigned int Protocoltype;          
    unsigned char HWsize;               
    unsigned char Protocolsize;         
    unsigned int ARP_Opcode;            
    unsigned char ARP_sourceMac[6];     
    unsigned long ARP_sourceIP;         
    unsigned char ARP_destMac[6];       
    unsigned long ARP_destIP;           
};
#define ARP_ANSWER      0
#define NO_ARP_ANSWER   !ARP_ANSWER

struct ARP_TABLE {
    unsigned long IP;
    unsigned char MAC[6];
};
////////////// IP ////////////////////////////////
struct IpHeader{
    unsigned char IP_Version_Headerlen;
    unsigned char IP_TOS;
    unsigned int IP_Totallenght;
    unsigned int IP_Identification;
    unsigned char IP_Flags;
    unsigned char IP_Fragmentoffset;
    unsigned char IP_TTL;
    unsigned char IP_Protocol;
    unsigned int IP_Headerchecksum;
    unsigned long IP_SourceIP;
    unsigned long IP_DestinationIP;
};
#define PROTO_ICMP 0x01
#define PROTO_TCP 0x06
#define PROTO_UDP 0x11
////////////// ICMP ///////////////////////////////
struct IcmpHeader{
	unsigned char ICMP_type;
	unsigned char ICMP_code;
	unsigned char ICMP_checksumByteOne;
	unsigned char ICMP_checksumByteTwo;
};

#define String StringBig
/////// DEBUG /////////////
//#define IsDEBUG	// auch im Paket "Debug" schalten
	#define DEBUG_BAUD 500000
	#define DEBUG_UART_NR 0
	#include "debugUart.h"

class Ethernet : public ENC28J60
{

public:
	//	configEthernet()
	//	PE:
	//		maxFrameLength = maximale Paketgre
	void configEthernet(uint16_t maxFrameLength=400);
	//	workEthernet()
	//	Bearbeitet Pakete.
	//	Muss von auen aufgerufen werden, bei INT vom Ethernet oder Poling oder Timer.
	void workEthernet();
	//	workOnTimer10ms()
	//	von auen aufgerufen bei Timer INT alle 100ms
	virtual void workOnTimer10ms();
	//	Die eigene IP-Adresse.
	uint32_t myIP;
	//	Die eigene Subnetzmaske.
	uint32_t myNetworkmask;
	//	Konstruktor
	Ethernet();

	//	Destruktor
	~Ethernet();

private:

protected:
	//	changeEndian16bit()
	//	Vertauscht Low-Byte mit High-Byte einer 16-Bit-Zahl
	uint16_t changeEndian16bit(uint16_t value);
	//	changeEndian32bit()
	//	Vertauscht Low-Byte mit High-Byte einer 32-Bit-Zahl
	uint32_t changeEndian32bit(uint32_t value);
	//	checksum16bit()
	//	Berechnet eine 16-Bit Prfsumme.
	//	
	//	PE:
	//		pointer = Zeiger auf Daten
	//		length = Anzahl de Byte
	uint16_t checksum16bit(uint8_t* pointer, uint16_t lenght);
	//	getFrame()
	//	Holt einen Ethernet-Frame.
	//	
	//	
	//	PA:
	//		Lnge
	unsigned int getFrame(unsigned int maxlen, unsigned char *ethernetbuffer);
	//	getIpFromMac()
	//	Erfragt die MAC-Adresse zu einer IP - Adresse.
	//	
	//	PE:
	//		ip = IP-Adresse zu der die MC-Adresse gesucht wird
	//		macBuffer = Buffer zur ablage der gefunden MAC-Adresse
	//		
	//	PA:
	//		- ARP_ANSWER = Die MAC anfrage war erfolgrich, Daten im macBuffer
	//		- NO_ARP_ANSWER = Die MAC-Anfrage ist Fehlgeschlagen.
	unsigned int getIpFromMac(unsigned long ip, unsigned char * macBuffer);
	//	makeEthHeader()
	//	Erstellt den richtigen Ethernetheader zur passenden Verbindung die gerade mit TCP_socket gewhlt ist
	//	Wird fr TCP und UDP bentigt.
	void makeEthHeader(uint8_t* MACadress , uint8_t * ethernetbuffer);
	//	makeIpHeader()
	//	Baut den IP-Header, es wird die Datengre ohne IP-Header und ein Pointer auf den Buffer bergeben
	//	Wird fr TCP und UDP bentigt.
	void makeIpHeader(uint32_t SourceIP, uint8_t Protocoll, uint16_t lenght , uint8_t*ethernetBuffer);
	//	onEncError()
	//	wird aufgerufen, wenn ENC nicht reagiert. ACHTUNG dies wird im Interrupt aufgerufen.
	virtual void onEncError();
	//	onArpPacket()
	//	Wird ausgelst, wenn ein ARP-Paket empfangen wurde.
	//	Das Address Resolution Protocol (ARP) ist ein Netzwerkprotokoll, das die Zuordnung von Netzwerkadressen zu Hardwareadressen mglich macht.
	virtual void onArpPacket(unsigned int lenght, unsigned char *ethernetBuffer);
	//	onIpPacket()
	//	Wird ausgelst, wenn ein IP-Paket empfangen wurde.
	//	
	//	PE:
	//		lenght = Anzahl der Bytes
	//		ethernetBuffer = Zeiger auf Daten
	virtual void onIpPacket(unsigned int lenght, unsigned char *ethernetBuffer);
	//	onIpPaketIcmp()
	//	Wird ausgelst, wenn ein IP/ICMP-Paket (PING) empfangen wurde.
	//	
	//	PE:
	//		lenght = Anzahl der Bytes
	//		ethernetBuffer = Zeiger auf Daten
	virtual void onIpPaketIcmp(unsigned int lenght, unsigned char *ethernetBuffer);
	//	onTimer1s()
	//	von innen aufgerufen, nach 100mal workOnTimer1()
	virtual void onTimer1s();
	//	sendFrame()
	//	Sendet ein Ethernet-Frame.
	void sendFrame(unsigned int packet_lenght, unsigned char *ethernetbuffer);
	//	eine ARP-Tabelle
	ARP_TABLE arpTable;
	//	Zhler, wieviel Bytes bertragen wurden
	uint32_t byteCounter;
	//	dient zum sperren das der work-Funktion, wenn z.B. gerade eine Frame gesendet wird
	uint8_t locked;
	uint16_t maxFrameLength;
	uint32_t packetCounter;

};

#endif
