///<ObjektNummer>1937</ObjektNummer>

#if !defined(h_EthernetTcp)
#define h_EthernetTcp

// Defines
#define F_CPU 20000000
#define MCU ATMEGA644P

#include <avr\io.h>
#include "Ethernet.h"

#include "Mx8_Timer.h"
#include "M8_Mx8_SPI.h"

	#define	RAM		0
	#define FLASH	1
	#define EEPROM	2


	struct IpPseudoHeader {
			volatile unsigned long IP_SourceIP;			// Quell IP
			volatile unsigned long IP_DestinationIP;		// Ziel IP
			volatile unsigned char IP_ZERO;				// muss 0 sein
			volatile unsigned char IP_Protokoll;			// bei TCP immer 6
			volatile unsigned int IP_TCP_lenght;			// lnge des Nutzdaten Segmentes ( TCP_Segment )
	};

	#define MAX_LISTEN_PORTS 4					// Anzahl der Port auf den gelauscht wird
	#define TCP_Port_not_use 0

	struct TcpPort {							// Struktur fr Ports
			volatile unsigned int TCP_Port;
	};

	// Struktur für den Connection SOocket. Hier werden die Sockets für die TCP-Verbindungen angelegt und den wichtige Werte gespeichert.
	// damit kann man jede verbindung genau nachverfolgen und auch halten.

	#define MAX_TCP_CONNECTIONS			4	// maximale gleichzeitige verbindungen
	#define MAX_TCP_RETRANSMISSIONS		5
	#define MAX_RECIVEBUFFER_LENGHT 	64	// maximale TCP-Empfangsbuffer Lnge
	#define CLOSETIMEOUT				200			// angabe in 1/100 sec
	#define RETRANSMISSIONTIMEOUT		500			// angabe in 1/100 sec
														// der timeout für retransmission
	#define CONNECTTIMEOUT				500				// angabe in 1/100 sec
	#define TimeOutCounter 				20				// angabe in 1 sec
														// timeout für das socket wie lange keine daten mehr ausgetauscht wurden

	struct TcpSocket {
			volatile unsigned char ConnectionState;				// 1 Byte
			volatile unsigned char SendState;					// 1 Byte
			volatile unsigned int SourcePort;					// 2 byte
			volatile unsigned int DestinationPort;				// 2 Byte
			volatile unsigned long SourceIP;					// 4 Byte
			volatile unsigned long SequenceNumber;				// 4 Byte
			volatile unsigned long AcknowledgeNumber;			// 4 Byte
			volatile unsigned int Windowsize;					// 2 Byte
			volatile unsigned int SendetBytes;					// 2 Byte
			volatile unsigned int Timeoutcounter;				// 2 Byte
			volatile unsigned char MACadress[6];				// 6 Byte
			volatile unsigned int Bufferlenght;					// 2 Byte = 32 Byte + Bufferlenght
			volatile unsigned char Recivebuffer[ MAX_RECIVEBUFFER_LENGHT ];
	};

	// Definitionen fr die ConnectionState
	#define SOCKET_NOT_USE		0x00			// SOCKET ist Frei
	#define SOCKET_SYNINIT		0x01			// SOCKET SYN init
	#define SOCKET_WAIT2SYNACK	0x04			// SOCKET hat SYN & ACK gesendet und wartet auf ACK
	#define SOCKET_SYNACK_OK	0x05			// SOCKET hat SYN+ACK nachen einen SYN empfangen ( fuer berbindungsaufbau )
	#define SOCKET_READY2USE	0x0F			// SYN erfolgreich und Socket bereit zur übernahme durch den Anwender
	#define SOCKET_READY		0x10			// Socket ist Bereit zur Benutzung
	#define SOCKET_WAIT2FINACK	0x81			// SOCKET wartet auch das FIN + ACK nach einem FIN + ACK
	#define SOCKET_WAIT2FIN		0x82			// SOCKET wartet auch das ACK nach einem FIN + ACK
	#define SOCKET_RESET		0x83			// SOCKET empfängt RST ( Reset )
	#define SOCKET_CLOSE		0xff			// SOCKET geschlossen

    #define NO_SOCKET_USED 		0xffff           // Rueckgabewert

	// Definitionen fr die SendState
	#define SOCKET_READY2SEND	0x00			// Socket bereit zum senden
	#define SOCKET_DATASENDED	0x80			// Socket hat Daten gesendet aber Window ist nicht voll und wartet auf ACK für Daten
	#define SOCKET_BUSY			0xff			// Socket hat Daten gesendet und warten auf ACK für Daten

	// der TCP-header
	#define MAX_TCP_Datalenght 	64
	struct TcpHeader {
			volatile unsigned int TCP_SourcePort;
			volatile unsigned int TCP_DestinationPort;
			volatile unsigned long TCP_SequenceNumber;
			volatile unsigned long TCP_AcknowledgeNumber;
			volatile unsigned char TCP_DataOffset;
			volatile unsigned char TCP_ControllFlags;
			volatile unsigned int TCP_Window;
			volatile unsigned int TCP_Checksum;
			volatile unsigned int TCP_UrgentPointer;
			volatile unsigned char TCP_Options[4];
	};

	// die einzelnen Controllflags des TCP-headers
	#define  TCP_NON_FLAG			0x00		// kein Flag gesetzt
	#define  TCP_FIN_FLAG			0x01		// Verbindung beenden gesenden/empfangen
	#define  TCP_SYN_FLAG			0x02		// Verbindung aufbauen gesenden/empfangen
	#define  TCP_RST_FLAG			0x04		// Reset gesenden/empfangen
	#define  TCP_PSH_FLAG			0x08		// Push gesendet/empfangen
	#define  TCP_ACK_FLAG			0x10        // Acknowledge gesendet/empfangen
	#define  TCP_URG_FLAG			0X20		// Urgentflag fr Dringlichkeit gesetzt

#define String StringBig
/////// DEBUG /////////////
//#define IsDEBUG	// auch im Paket "Debug" schalten
	#define DEBUG_BAUD 500000
	#define DEBUG_UART_NR 0
	#include "debugUart.h"

class EthernetTcp : public Ethernet
{

public:

	//	checkPortInList()
	uint16_t checkPortInList(uint16_t port);
	//	checkPortRequest()
	//	Prft ob an einem Port eine Verbindung eingegangen ist.
	uint16_t checkPortRequest(uint16_t port);
	//	checkSocketState()
	//	Gibt den Status des Socket zurck.
	uint16_t checkSocketState(uint16_t socket);
	//	closeTcpSocket()
	//	Schliesst ein Socket und Beendet die TCP-Verbindung.
	void closeTcpSocket(uint16_t socket);
	//	copyTCPdata2socketbuffer()
	//	Kopiert die Daten auf den TCP-packet in den Socketpuffer
	uint16_t copyTCPdata2socketbuffer(uint16_t socket, uint16_t lenght , uint8_t *ethernetBuffer);
	//	getFreeSocket()
	//	Holt das naechste freie Socket
	uint16_t getFreeSocket();
	//	getSocketData()
	//	Holt empfangende Daten aus den Socketpuffer.
	//	
	//	PE:
	//		buffer = Ziel, wenn NULL wird der Inhalt nur verworfen
	//		
	//	PA:
	//		Anzahl der kopierten Bytes.
	uint16_t getSocketData(uint16_t socket , uint16_t bufferlen, uint8_t *buffer);
	//	getSocketDataNextLine()
	//	Holt empfangende Daten bis zum "\r\n" aus den Socketpuffer.
	//	
	//	
	//	PA:
	//		Anzahl der kopierten Bytes.
	uint16_t getSocketDataNextLine(uint16_t socket , uint16_t bufferlen, uint8_t *buffer);
	//	putSocketData()
	//	Sendet Daten ueber ein Socket aus dem RAM/FLASH
	//	
	//	PE:
	//		memMode = RAM|FLASH
	uint8_t putSocketData(uint16_t socket, uint8_t* sendbuffer, uint8_t memMode);
	//	registerSocket()
	//	Reistriert einen neuen Socket wenn noch einer frei ist und sichert alle ntigen Daten in ihn
	uint16_t registerSocket(uint8_t* ethernetBuffer);
	//	registerTcpPort()
	//	
	//	PA:
	//		true wenn ok
	uint8_t registerTcpPort(uint16_t port);
	//	putSocketData()
	//	Sendet Daten ueber ein Socket aus dem RAM/FLASH
	//	
	//	PE:
	//		memMode = RAM|FLASH
	uint8_t putSocketData(uint16_t socket, uint16_t lenght, uint8_t* sendbuffer, uint8_t memMode);
	//	Konstruktor
	EthernetTcp();

	//	Destruktor
	~EthernetTcp();

private:
	//	putSocketData2()
	//	wird von putSocketData verwendet
	uint8_t putSocketData2(uint16_t socket, uint16_t lenght, uint8_t* sendbuffer, uint8_t memMode);

protected:

	//	getSocket()
	uint16_t getSocket(uint8_t* ethernetBuffer);

	//	makeTcpHeaderAndSend()
	void makeTcpHeaderAndSend(uint16_t socket, uint8_t TCP_flags, uint16_t lenght, uint16_t windowsize, uint8_t* ethernetBuffer);
	//	onIpPacket()
	//	Wird ausgelst, wenn ein IP-Paket empfangen wurde.
	//	
	//	PE:
	//		lenght = Anzahl der Bytes
	//		ethernetBuffer = Zeiger auf Daten
	virtual void onIpPacket(unsigned int lenght, unsigned char *ethernetBuffer);
	//	onIpPacketTcp()
	//	Wird ausgelst, wenn ein IP-Paket empfangen wurde.
	//	
	//	PE:
	//		lenght = Anzahl der Bytes
	//		ethernetBuffer = Zeiger auf Daten
	virtual void onIpPacketTcp(unsigned int lenght, unsigned char *ethernetBuffer);
	//	onTimer1s()
	//	von innen aufgerufen, nach 100mal workOnTimer1()
	virtual void onTimer1s();

	//	timeOutHandler()
	void timeOutHandler();
	TcpPort portTable[MAX_LISTEN_PORTS];
	uint16_t rxErrorCounter;
	TcpSocket volatile tcpSocketTable[MAX_TCP_CONNECTIONS];
	uint16_t txErrorCounter;

};

#endif
